@@ -60,6 +60,38 @@ impl From<&Fprint> for Fingerprint {
     }
 }
 
+/// A `BundledChunk` is sent with a manifest, instead of indirecting
+/// through a content-addressed chunk store.
+///
+/// Readers may assume bundled data matches the list of chunk
+/// fingerprints in the `Manifest`.  They do not have to handle
+/// internally inconsistent `BundledChunk`s nor disagreements
+/// between `BundledChunk`s and the rest of the manifest.
+///
+/// In other words, bundling chunks with a manifest isn't merely a
+/// best-effort optimisation: incorrect metadata (`chunk_index` and
+/// `chunk_offset` in particular) can lead to an invalid snapshot.
+#[derive(Clone, PartialEq, Eq, prost::Message)]
+pub struct BundledChunk {
+    // The bundled chunk is expected to match this index in the
+    // manifest's list of chunk fingerprints.
+    #[prost(uint64, tag = "1")]
+    pub chunk_index: u64,
+
+    // The bundled chunk should be at that byte offset in the
+    // manifest's snapshot.
+    #[prost(uint64, tag = "2")]
+    pub chunk_offset: u64,
+
+    // The chunk's data has this fingerprint.
+    #[prost(message, tag = "3")]
+    pub chunk_fprint: Option<Fprint>,
+
+    // And the chunk consists of these bytes.
+    #[prost(bytes, tag = "4")]
+    pub chunk_data: Vec<u8>,
+}
+
 #[derive(Clone, PartialEq, Eq, prost::Message)]
 pub struct ManifestV1 {
     // The fingerprint for the file's 100-byte sqlite header.  There
@@ -130,6 +162,13 @@ pub struct ManifestV1 {
     // do so just before serialising the manifest.
     #[prost(fixed64, repeated, tag = "15")]
     pub chunks: Vec<u64>,
+
+    // The list of chunks bundled with this manifest.  When we have
+    // such a chunk, we don't look for it in the content-addressed
+    // store, and we can't assume the corresponding chunk exists in
+    // the content-addressed store.
+    #[prost(message, repeated, tag = "16")]
+    pub bundled_chunks: Vec<BundledChunk>,
 }
 
 /// When deserialising a `Manifest`, it usually makes sense to use
@@ -620,10 +659,14 @@ pub(crate) fn extract_manifest_len(manifest: &Manifest) -> Result<u64> {
 /// Returns the list of chunks in the manifest proto at `Path`, along
 /// with the `base_chunks_fprint` if any, or an empty list if there is
 /// no such file.
+///
+/// All fingerprints in the list of chunks that may be satisfied by
+/// the bundled chunks are replaced with `substitute_bundled_fprints`.
 #[instrument]
 pub(crate) fn parse_manifest_chunks(
     path: &std::path::Path,
     replication_targets: &[ReplicationTarget],
+    substitute_for_bundled_fprints: Fingerprint,
 ) -> Result<(Vec<Fingerprint>, Option<Arc<Chunk>>)> {
     match std::fs::read(path) {
         Ok(contents) => {
@@ -633,7 +676,24 @@ pub(crate) fn parse_manifest_chunks(
                 Some(replication_targets),
                 path,
             )?;
-            Ok((extract_manifest_chunks(&manifest)?, base_or))
+            let mut chunks = extract_manifest_chunks(&manifest)?;
+
+            // Overwrite bundled chunks with the substitution fprint.
+            if let Some(v1) = manifest.v1.as_ref() {
+                for bundled in v1.bundled_chunks.iter() {
+                    if let Some(fprint) = bundled.chunk_fprint.as_ref() {
+                        if bundled.chunk_index < chunks.len() as u64 {
+                            let index = bundled.chunk_index as usize;
+
+                            if chunks[index] == fprint.into() {
+                                chunks[index] = substitute_for_bundled_fprints;
+                            }
+                        }
+                    }
+                }
+            }
+
+            Ok((chunks, base_or))
         }
         Err(e) if e.kind() == std::io::ErrorKind::NotFound => Ok((Vec::new(), None)),
         Err(e) => Err(chain_error!(e, "failed to open manifest proto file", ?path)),
